/**********************************************************************************
 * Project: GSM - Electroweak fitting package                                     *
 * Package: GSM                                                                   *
 * Class  : MW                                                                    *
 *                                                                                *
 * Description:                                                                   *
 *      Prediction of W mass                                                      *
 *                                                                                *
 * Papers:                                                                        *
 *      hep-ph/0311148 by Awramik et. al.                                         *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Martin Goebel   <martin.goebel@desy.de>  - DESY, Germany                  *
 *      Andreas Hoecker <Andreas.Hocker@cern.ch> - CERN, Switzerland              *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: MW.cxx,v 1.33 2008/07/29 14:37:41 mgoebel Exp $
 **********************************************************************************/
#include "TMath.h"
#include "TString.h"

#include "Gfitter/GMath.h"
#include "Gfitter/GTheory.h"
#include "Gfitter/GTheoryRef.h"
#include "Gfitter/GParameterRef.h"
#include "Gfitter/GReference.h"
#include "Gfitter/GVariable.h"
#include "Gfitter/GStore.h"
#include "Gfitter/GConstants.h"

#include "GSM/MW.h"
#include "GSM/DAlphaQED.h"
#include "GSM/AlphaQCDAtQ.h"
#include "GSM/MH.h"

using namespace Gfitter;

ClassImp(GSM::MW)

GSM::MW::MW()
   : Gfitter::GTheory()
{
   SetTheoryName( GetName() );
   SetExistDerivative( kFALSE );
  
   const TString& logMH = gStore()->GetVariable( "GSMFlags::logMH" )->GetStringValue();
   m_logger << kINFO << "Using logMH: \"" << logMH << "\"" << GEndl;
   
   if      (logMH == "Yes" ) m_logMH  = kTRUE;
   else if (logMH == "No" )  m_logMH  = kFALSE;
   else {
      m_logger << kFATAL << "unknown value for \"GSMFlags::logMH\": \"" << logMH << "\""
               << ". Possible are: \"Yes\" and \"No\"\""
               << GEndl;
   }

   BookParameter( "MZ",                   &p_MZ );   
   BookParameter( "mt",                   &p_mt );   
   BookParameter( "DeltaMW_Scale",        &p_DeltaMW_Scale );   
   BookParameter( "GF",                   &p_GF );   
   BookTheory   ( "GSM::AlphaQCDAtQ/MZ",  &t_AlphasMZ );   
   BookTheory   ( "GSM::DAlphaQED",       &t_DAlphaQED );   
   BookTheory   ( "GSM::MH" ,             & t_MH );
}

// init coefficients
void GSM::MW::Initialise()
{
   m_MW0 = 80.3799;
   m_c1  = 0.05429;
   m_c2  = 0.008939;
   m_c3  = 0.0000890;
   m_c4  = 0.000161;
   m_c5  = 1.070;
   m_c6  = 0.5256;
   m_c7  = 0.0678;
   m_c8  = 0.00179;
   m_c9  = 0.0000659;
   m_c10 = 0.0737;
   m_c11 = 114.9;
}

// eq. (6) of hep-ph/0311148
Double_t GSM::MW::GetTheoryPrediction()
{
   Double_t MH      = GetMH().GetValue(); //p_MH;
   if( m_logMH ) MH = TMath::Exp( GetMH().GetValue() ); //p_MH );

   Double_t deAlpha     = (GetDAlphaQED().DAlphaQEDMZ()/0.05907) - 1.0;
   Double_t deAlphaS    = (GetAlphasMZ()/0.119) - 1.0;   

   Double_t LH          = TMath::Log(MH/100);
   Double_t dH          = Gfitter::GMath::IPow( MH/100.0 , 2);

   Double_t dt          = ((p_mt/174.3)*(p_mt/174.3) - 1.0);
   Double_t dZ          = (p_MZ/91.1875) - 1.0;

   Double_t mw = ( m_MW0 - m_c1*LH - m_c2*LH*LH + m_c3*Gfitter::GMath::IPow(LH,4) 
                   + m_c4*(dH-1) - m_c5*deAlpha 
                   + m_c6*dt - m_c7*dt*dt - m_c8*LH*dt 
                   + m_c9*dH*dt - m_c10*deAlphaS + m_c11*dZ );
  
   // theoretical uncertainty in MW
   mw = mw + p_DeltaMW_Scale*0.004;

   // test GF dependency of GF
   if( false ){
      Double_t MZ2 = p_MZ*p_MZ;

      Double_t deltaMW2 = ( MZ2/2.*( TMath::Sqrt(1-TMath::Sqrt(8.)*TMath::Pi()*GConstants::alphaQED()/(MZ2*GConstants::GF())) 
                                           - TMath::Sqrt(1-TMath::Sqrt(8.)*TMath::Pi()*GConstants::alphaQED()/(MZ2*p_GF)) ) );
      mw = TMath::Sqrt(mw*mw + deltaMW2);
                            
   }
   
   return mw;
}
