/**********************************************************************************
 * Project: GSM - Electroweak fitting package                                     *
 * Package: GSM                                                                   *
 * Class  : QPoleMass                                                             *
 *                                                                                *
 * Description:                                                                   *
 *      Auxiliary Theory for inverting Ms-bar masses to pole quark masses         *
 *                                                                                *
 * Papers:                                                                        *
 *      hep-ph/9911434                                                            *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Martin Goebel   <martin.goebel@desy.de>  - DESY, Germany                  *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: QPoleMass.cxx,v 1.8 2008/06/15 21:18:09 hoecker Exp $
 **********************************************************************************/
#include "TMath.h"

#include "Gfitter/GMath.h"
#include "Gfitter/GParameterRef.h"
#include "Gfitter/GTheoryRef.h"
#include "Gfitter/GVariable.h"
#include "Gfitter/GStore.h"

#include "GSM/QPoleMass.h"
#include "GSM/RunningAlphaQCD.h"
#include "GSM/AlphaQCDAtQ.h"

using namespace Gfitter;

ClassImp(GSM::QPoleMass)

GSM::QPoleMass::QPoleMass()
   : GAuxTheory()
{
   SetTheoryName( GetName() );
   SetExistDerivative( kFALSE );
   
   BookParameter( "mu_MSb", &p_mu );   
   BookParameter( "md_MSb", &p_md );   
   BookParameter( "ms_MSb", &p_ms );   
   BookParameter( "mc_MSb", &p_mc );   
   BookParameter( "mb_MSb", &p_mb ); 

   BookTheory( "GSM::RunningAlphaQCD", & t_AlphasRun );
}

// MS_bar to pole mass
// eq. (34)  of hep-ph/9911434 
Double_t GSM::QPoleMass::MSMassToPoleMass( Double_t m, Int_t nf )
{
   Double_t As        = 0;
   Double_t nl        = nf - 1;

   switch( nf ){
   case 1:
      As  = GetAlphasRun().EvolveAlphas(1)/TMath::Pi();
      break;
   case 2:
      As  = GetAlphasRun().EvolveAlphas(1)/TMath::Pi();
      break;
   case 3:
      As  = GetAlphasRun().EvolveAlphas(1)/TMath::Pi();
      break;
   case 4:
      if (TMath::IsNaN(p_mc)) m_logger << kFATAL << "<QPoleMass::MSMassToPoleMass> p_mc is NaN !" << GEndl;
      As  = GetAlphasRun().EvolveAlphas(p_mc)/TMath::Pi();
      break;
   case 5:
      if (TMath::IsNaN(p_mb)) m_logger << kFATAL << "<QPoleMass::MSMassToPoleMass> p_mb is NaN !" << GEndl;
      As  = GetAlphasRun().EvolveAlphas(p_mb)/TMath::Pi();
      break;
   default:
      m_logger << kFATAL << "<PoleMassToMSMass> Unknown flavour: " << nf << GEndl;
   }  
   
   return m*( 1.0 + 1.333*As 
              + As*As*( 13.44 - 1.041*nl )
              + As*As*As*( 194.0 - 27.0*nl + 0.653*nl*nl) );
}
