/**********************************************************************************
 * Project: GSM - Electroweak fitting package                                     *
 * Package: GSM                                                                   *
 * Class  : RadiatorFunctions                                                     *
 *                                                                                *
 * Description:                                                                   *
 *      Auxiliary Theory computes QCD radiator functions                          *
 *      for Z and W decay                                                         *
 *                                                                                *
 * References:                                                                    *
 *      The Standard Model in the Making, Oxford 1999                             *
 *      Bardin et al, CPC 133 (2001) 229-395, hep-ph/9908433                      *
 *      (Raditor function in dizet6_42.f line 5046-5227)                          *
 *      Baikov et al., PR Lett. 101(2008) 012002, arXiv:0709.1983                 *
 *                                                                                *
 * Authors:                                                                       *
 *      See gsm/AUTHORS file                                                      *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: RadiatorFunctions.h,v 1.10 2008/01/16 14:07:57 mgoebel Exp $
 **********************************************************************************/
#ifndef GSM_RadiatorFunctions
#define GSM_RadiatorFunctions

#include "Gfitter/GAuxTheory.h"
#include "Gfitter/GTypes.h"

class Gfitter::GReference;
class Gfitter::GParameterRef;
class Gfitter::GTheoryRef;

namespace GSM {

   class QMassRunning;
   class DAlphaQED;

   class RadiatorFunctions : public Gfitter::GAuxTheory {

   public:
      
      RadiatorFunctions();
      ~RadiatorFunctions() {}

      // initalize beta function
      virtual void Initialise();

      // dummy prediction
      Double_t GetTheoryPrediction() { return 0; }

      // accessors - vector and axial radiator functions
      Double_t GetRVq( Gfitter::GTypes::Particle particle, Double_t Charge );
      Double_t GetRAq( Gfitter::GTypes::Particle particle, Double_t Charge, Double_t T3 );

   protected:

      // update of flags in case of parameter change
      void UpdateLocalFlags( Gfitter::GReference& ref );

      // update paramters
      void Update();

      // parameters from the Data-Card
      Gfitter::GParameterRef p_MZ;
      Gfitter::GParameterRef p_mt;

      // reference to DAlphaQED
      DAlphaQED& GetDAlphaQED() const { return (DAlphaQED&)*t_DAlphaQED; }      
      Gfitter::GTheoryRef    t_DAlphaQED; 
      
      // reference to QMassRunning
      QMassRunning& GetQMassRun() const { return (QMassRunning&)*t_qMassRun; }      
      Gfitter::GTheoryRef    t_qMassRun;
   
      // reference to AlphasMZ
      Gfitter::GTheory& GetAlphasMZ() const { return (Gfitter::GTheory&)*t_AlphasMZ; }      
      Gfitter::GTheoryRef    t_AlphasMZ; 

      // scale parameter to estimate theoretical uncertainties: set to zero for NO efffect
      Gfitter::GParameterRef p_DeltaAlphasTheoC05_Scale;  // for C05 coefficient
      Gfitter::GParameterRef p_DeltaAlphasTheoCMt3_Scale; // for CMt3 coefficient
    
      // flag to compute radiator function to alpha_s NLO only
      Bool_t   m_useNLOAlphasOnly;

      // fixed order constants
      // massless corr.
      Int_t    m_nf1;
      Double_t m_C01;
      Double_t m_C02;
      Double_t m_C03;
      Double_t m_C04;
      Double_t m_C05;
      
      Double_t m_CL1;
      Double_t m_CL2;
      Double_t m_CL3;

      // vector
      Double_t m_CV1;
      Double_t m_CV2;
      Double_t m_CV3;
      
      Double_t m_CV40;
      Double_t m_CV41;
      Double_t m_CV42;
      
      Double_t m_CVL42;

      // axial
      Double_t m_CA0;
      Double_t m_CA1;
      Double_t m_CA2;
      Double_t m_CA3;

      Double_t m_CA40;
      Double_t m_CA41;
      Double_t m_CA42;

      Double_t m_CAL42;

      Double_t m_C42;
      Double_t m_Ct2;
      
      Double_t m_CMt2;
      Double_t m_CMt3;
      Double_t m_CMt4;
      
      // Update parmeters - intermediate parmeters
      Double_t m_asMZpi;
      Double_t m_aQEDMZpi;
      Double_t m_RatioMcMZ; 
      Double_t m_RatioMbMZ;  
      Double_t m_RatioMtMZ;  
      Double_t m_RatioMcMt;  
      Double_t m_RatioMbMt;  
   
      Double_t m_LightQu2;   
      Double_t m_LightQu4Mc; 
      Double_t m_LightQu4Mb; 

   private:

      // boolean flags to track recomputation
      Bool_t m_isUpToDate_Update;
      
      ClassDef(RadiatorFunctions,0)
   };
}

#endif
