/**********************************************************************************
 * Project: GSM - Electroweak fitting package                                     *
 * Package: GSM                                                                   *
 * Class  : WParametrisation                                                      *
 *                                                                                *
 * Description:                                                                   *
 *      W Boson decay / total width                                               *
 *                                                                                *
 * Sources:                                                                       *
 *      (i) arXiv: 1104.1769v1 Cho, Hagiwara, et al                               *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Martin Goebel   <martin.goebel@desy.de>  - DESY, Germany                  *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: WParametrisation.cxx,v 1.11 2007/10/25 13:07:28 haller Exp $
 **********************************************************************************/
#include "TMath.h"

#include "Gfitter/GMath.h"
#include "Gfitter/GConstants.h"
#include "Gfitter/GTheory.h"
#include "Gfitter/GTheoryRef.h"
#include "Gfitter/GParameterRef.h"
#include "Gfitter/GReference.h"
#include "Gfitter/GVariable.h"
#include "Gfitter/GStore.h"

#include "GSM/WParametrisation.h"

using namespace Gfitter;

ClassImp(GSM::WParametrisation)

GSM::WParametrisation::WParametrisation()
   : WBase(), 
     m_isUpToDate_Update( kFALSE )     
{
   SetTheoryName( GetName() );
   SetExistDerivative( kFALSE );
   
   const TString& logMH = gStore()->GetVariable( "GSMFlags::logMH" )->GetStringValue();
   m_logger << kINFO << "Using logMH: \"" << logMH << "\"" << GEndl;
   
   if      (logMH == "Yes" ) m_logMH  = kTRUE;
   else if (logMH == "No" )  m_logMH  = kFALSE;
   else {
      m_logger << kFATAL << "unknown value for \"GSMFlags::logMH\": \"" << logMH << "\""
               << ". Possible are: \"Yes\" and \"No\"\""
               << GEndl;
   }

   BookTheory   ( "GSM::MH"                    , &t_MH );
   BookTheory   ( "GSM::AlphaQCDAtQ/MZ"        , &t_AlphasMZ );   
}

void GSM::WParametrisation::UpdateLocalFlags( GReference& /* ref */ )
{
   m_isUpToDate_Update = kFALSE;
}

void GSM::WParametrisation::Update()
{
   if (m_isUpToDate_Update) return;

   // now, it is uptodate (I mean... it will be)
   m_isUpToDate_Update = kTRUE;

   Double_t MW     = GetMW();
   Double_t MH     = GetMH().GetValue(); 
   if (m_logMH) MH = TMath::Exp( GetMH().GetValue() );
   
   // eq. 22 and 23e from (i) 
   Double_t RW = 2.1940 - 0.16*(TMath::Log(1+std::pow(23./MH,2)) - TMath::Log(1+std::pow(23./100,2))); 
   Double_t xs = (GetAlphasMZ() - 0.118)/0.003;

   // eq. 20 from (i)
   m_gammaW = 0.33904*MW*MW*MW*p_GF*(1. + 8.478e-3*RW + 0.00065*xs);
  
   // now, parameters are up-to-date
   SetUpToDate();
}

