/**********************************************************************************
 * Project: GSM - Electroweak fitting package                                     *
 * Package: GSM                                                                   *
 * Class  : ZFitterBosonPart                                                      *
 *                                                                                *
 * Description:                                                                   *
 *      Auxiliary Theory for bosonic part of self energies                        *
 *      one loop core of ZFitter option                                           *
 *                                                                                *
 * References:                                                                    *
 *      The Standard Model in the Making                                          *
 *      Nucl. Phys. B197 (1982) 1-44 / first summary of one loop core             *
 *      hep-ph/9709299, hep-ph/9908433                                            *
 *      ZFITTER package dizet6_42.f                                               *
 *                                                                                *
 * Authors:                                                                       *
 *      See gsm/AUTHORS file                                                      *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: ZFitterBosonPart.cxx,v 1.9 2008/05/18 15:51:12 mgoebel Exp $
 **********************************************************************************/
#include <math.h>

#include "TMath.h"

#include "Gfitter/GMath.h"
#include "Gfitter/GConstants.h"
#include "Gfitter/GTheory.h"
#include "Gfitter/GTheoryRef.h"
#include "Gfitter/GParameterRef.h"
#include "Gfitter/GReference.h"
#include "Gfitter/GVariable.h"
#include "Gfitter/GStore.h"

#include "GSM/ZFitterBosonPart.h"
#include "GSM/ZFitterMath.h"
#include "GSM/MH.h"

using std::complex;

using namespace Gfitter;

GSM::ZFitterBosonPart::ZFitterBosonPart() 
   : Gfitter::GAuxTheory(),
     m_isUpToDate_Update( kFALSE )   
{ 
  
   SetTheoryName( GetName() );
   SetExistDerivative( kFALSE );

   const TString& logMH = gStore()->GetVariable( "GSMFlags::logMH" )->GetStringValue();
   m_logger << kINFO << "Using logMH: \"" << logMH << "\"" << GEndl;
   
   if      (logMH == "Yes" ) m_logMH  = kTRUE;
   else if (logMH == "No" )  m_logMH  = kFALSE;
   else {
      m_logger << kFATAL << "unknown value for \"GSMFlags::logMH\": \"" << logMH << "\""
               << ". Possible are: \"Yes\" and \"No\"\""
               << GEndl;
   }

   BookParameter( "MZ"      , & p_MZ );
   BookTheory   ( "GSM::MW" , & t_MW );
   BookTheory   ( "GSM::MH" , & t_MH );
}

void GSM::ZFitterBosonPart::UpdateLocalFlags( GReference& /* ref */ )
{
   m_isUpToDate_Update = kFALSE;
}

void GSM::ZFitterBosonPart::Update()
{
   if (m_isUpToDate_Update) return;

   // now, it is uptodate (I mean... it will be)
   m_isUpToDate_Update = kTRUE;

   Double_t MH      = GetMH().GetValue(); //p_MH;
   if( m_logMH ) MH = TMath::Exp(GetMH().GetValue() ); //p_MH );

   Double_t MZ2 = p_MZ*p_MZ;
   Double_t MW2 = GMath::IPow( GetMW(), 2 );
   Double_t MH2 = MH*MH;
  
   m_R         = MW2/MZ2;
   m_rw        = MH2/MW2;
   m_rz        = MH2/MZ2;
   
   // Get L and J functions
   // see dizet6_42.f line 1962-1967
   m_L0 = ZFitterMath::L( -MZ2, MW2, MW2 )/MZ2;
   m_L1 = ZFitterMath::L( -MW2, MH2, MW2 )/MW2;
   m_L2 = ZFitterMath::L( -MW2, MW2, MZ2 )/MW2;
   m_L3 = ZFitterMath::L( -MZ2, MH2, MZ2 )/MW2;
   m_L4 = ZFitterMath::L( -MZ2, MW2, MW2 )/MW2;
   m_J1 = ZFitterMath::J( -MW2, MH2, MW2 )*MH2;
   m_J3 = ZFitterMath::J( -MZ2, MH2, MZ2 )*MH2/m_R;

   SetUpToDate();
} 

// see to all equation for a deeper understanding
// The Standard Model in the Making page 192-195
// hep-ph/9908433v3 page 152-155 and entire Appendix A
// Nucl. Phys. B197 (1982) 1-44

// eq.(259) of  hep-ph/9709229v1
// Z boson self energy at MZ
// see dizet6_42.f line 1977
complex<Double_t> GSM::ZFitterBosonPart::GetZbAtMZ()
{
   Update(); 
   return ( 35.0/(18.0*m_R) + 35.0/18.0 - 34.0/3.0*m_R - 8.0*m_R*m_R - m_rw/2.0
            + m_rw*m_rw*m_R/12.0 + m_rw*(-3.0/4.0 + m_rz/4.0 - m_rz*m_rz/24.0)*TMath::Log(m_rz)
            + 5.0/(6.0*m_R)*TMath::Log(m_R) + (0.5 -m_rz/6.0 + m_rz*m_rz/24.0)*m_L3
            + (1.0/24.0 + 2.0/3.0*m_R -17/6.0*m_R*m_R - 2.0*m_R*m_R*m_R)*m_L4 );
}

// eq.(260) of  hep-ph/9709229v1
// derivative of Z boson self energy at MZ
// see dizet6_42.f line 1983
complex<Double_t> GSM::ZFitterBosonPart::GetZbFAtMZ()
{         
   Update(); 
   return ( -4.0*m_R*m_R + 17.0/3.0*m_R - 23.0/9.0 + 5.0/(18.0*m_R) - m_rw/2 
            + m_rw*m_rz/6.0 - TMath::Log(m_R)/(12.0*m_R)
            + m_rw*(-3.0/4.0 + 3.0/8.0*m_rz - m_rz*m_rz/12.0)*TMath::Log(m_rz) + 0.5/m_R*TMath::Log(m_rz)
            + (-m_R*m_R*m_R + 7.0/6.0*m_R*m_R - 17.0/12.0*m_R - 1.0/8.0)*m_L4
            + (0.5 -5.0/24.0*m_rz + 1.0/12.0*m_rz*m_rz)*m_L3 + 0.5*m_J3 );   
}

// eq.(257) of  hep-ph/9709229v1
// W boson self ennergy at 0 GeV
// see dizet6_42.f line 1969
complex<Double_t> GSM::ZFitterBosonPart::GetWbAt0()
{         
   Update(); 
   return ( 5.0/(8.0*m_R) - 17.0/4.0 + 5.0/8.0*m_R*(1.0+m_R) - m_rw/8.0
            + 3.0/4.0*m_rw/(1-m_rw)*TMath::Log(m_rw) + (3.0/(4.0*m_R) + 9.0/4.0 - 3.0/(1-m_R))*TMath::Log(m_R) );
}


// eq.(258) of  hep-ph/9709229v1
// W boson self ennergy at MW
// see dizet6_42.f line 1972
complex<Double_t> GSM::ZFitterBosonPart::GetWbAtMW()
{   
   Update();
   return ( 1.0/(12.0*m_R*m_R) + 23.0/(12.0*m_R) - 157.0/9.0 - m_rw/2.0 + m_rw*m_rw/12.0
            - m_rw*(3.0/4.0 - m_rw/4.0 + m_rw*m_rw/24.0)*TMath::Log(m_rw)
            + (1.0/(24.0*m_R*m_R*m_R) + 7.0/(12.0*m_R*m_R) - 7.0/(2.0*m_R))*TMath::Log(m_R)
            + (0.5 -m_rw/6.0 + m_rw*m_rw/24.0)*m_L1
            + (1.0/(24.0*m_R*m_R) + 2.0/(3.0*m_R) - 17.0/6.0 - 2.0*m_R)*m_L2 );
}

// eq.(A.7) of Nucl. Phys. B197 (1982)
// fermionic and bosonic part were added together in that eq.
// photon Z mixing function
// see dizet6_42.f line 1988
complex<Double_t> GSM::ZFitterBosonPart::GetMbPhoZAtMZ() 
{
   Update(); 
   return ( 2.0/(9.0*m_R) + 35.0/18.0 - 34.0/3.0*m_R - 8.0*m_R*m_R
            + (1.0/24.0 + 2.0/3.0*m_R - 17.0/6.0*m_R*m_R - 2.0*m_R*m_R*m_R)*m_L4 );
}

// eq.(A.3) of Nucl. Phys. B197 (1982)
// fermionic and bosonic part were added together in that eq.
// derivative of W boson self energy at MW
// see dizet6_42.f line 1990
complex<Double_t> GSM::ZFitterBosonPart::GetWbFAtMW() 
{
   Update(); 
   return ( m_R - 34/9.0 + 2/m_R + 1/(6.0*m_R*m_R) - m_rw/2.0 + m_rw*m_rw/6.0
            + (3.0*m_R + 5/2.0 - 17/(4.0*m_R) + 7/(8.0*m_R*m_R) + 1/(12.0*m_R*m_R*m_R))*TMath::Log(m_R) 
            + (0.5 - 3.0*m_rw/4.0 + 3.0*m_rw*m_rw/8.0 - GMath::IPow(m_rw,3)/12.0)*TMath::Log(m_rw)
            + (-m_R/2.0 - 2.0 + 25/(24.0*m_R) + 1/(12.0*m_R*m_R))*m_L2
            + (0.5 - 5*m_rw/24.0 + m_rw*m_rw/12.0)*m_L1 + 0.5*m_J1 ); 
}
