/**********************************************************************************
 * Project: GSM - Electroweak fitting package                                     *
 * Package: GSM                                                                   *
 * Class  : ZFitterEW2Loop                                                        *
 *                                                                                *
 * Description:                                                                   *
 *      Electroweak two loop calculation for ew formfactors                       *
 *      of order O(alpha^2*mt^2/MW^2)                                             *
 *                                                                                *
 * Source:                                                                        *
 *      hep-ph/9905472, hep-ph/9603374                                            * 
 *      The Standard Model in the Making, Oxford 1999                             *
 *      Translation of Degrassi's and Gambino's m2tcor5_11.f package from ZFitter *
 *                                                                                *
 * Authors:                                                                       *
 *      See gsm/AUTHORS file                                                      *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: ZFitterEW2Loop.h,v 1.6 2008/05/18 15:51:12 mgoebel Exp $
 **********************************************************************************/
#ifndef GSM_ZFitterEW2Loop
#define GSM_ZFitterEW2Loop

#include <complex>
#include "Gfitter/GAuxTheory.h"

class Gfitter::GReference;
class Gfitter::GParameterRef;
class Gfitter::GTheoryRef;

namespace GSM {

   class MH;

   class ZFitterEW2Loop : public Gfitter::GAuxTheory {
    
   public:
   
      ZFitterEW2Loop();
      virtual ~ZFitterEW2Loop() {}
      
      // accessors, downcast for using them  
      Double_t GetDrho2L();
      Double_t GetDkap2L();
      Double_t GetDrdRem();
      Double_t GetDkdRem( Double_t pi3f );
      Double_t GetRhoRem( Double_t pi3f );
    

      void Initialise() {}
         
   protected:

      // update of flags in case of parameter change
      void UpdateLocalFlags( Gfitter::GReference& ref );
   
      // update parameters
      void Update();

      // updated values in Update function
      Int_t    m_Nc;
      Double_t m_MZ;
      Double_t m_c2;
      Double_t m_s2;
      Double_t m_mu;
      Double_t m_sman;
      Double_t m_zt;
      Double_t m_ht;
      Double_t m_xt;

      Double_t m_deltarw;
      Double_t m_deltarwOS;
      Double_t m_kappa;
      Double_t m_deltaeta;
      Double_t m_Drho2L;
      Double_t m_Dkap2L;
      Double_t m_DrdRem;
    
      // reference to W mass
      Gfitter::GTheory& GetMW() const { return (Gfitter::GTheory&)*t_MW; }      
      Gfitter::GTheoryRef    t_MW; 

      // reference to Higgs mass
      GSM::MH& GetMH() const{ return (GSM::MH&)*t_MH; }
      Gfitter::GTheoryRef t_MH;
      
      // vparameters from the DataCard
      Gfitter::GParameterRef p_MZ; 
      Gfitter::GParameterRef p_mt; 
      Gfitter::GParameterRef p_GF; 
     
      // functions for computing the 2 loop corrections
      Double_t deltarw   ( const Double_t& zt, const Double_t& ht, const Double_t& s2, const Double_t& mz, const Double_t& mu);
      Double_t ECharge   ( const Double_t& zt, const Double_t& ht, const Double_t& s2, const Double_t& mz, const Double_t& mu);
      Double_t deltarho  ( const Double_t& zt, const Double_t& ht, const Double_t& s2, const Double_t& mz, const Double_t& mu);
      Double_t deltarhoOS( const Double_t& zt, const Double_t& s2, const Double_t& mz, const Double_t& mu );
      Double_t deltarwOS ( const Double_t& zt, const Double_t& s2, const Double_t& mz, const Double_t& mu );
      Double_t kappa2L   ( const Double_t& zt, const Double_t& ht, const Double_t& c2, const Double_t& mz, const Double_t& mu );
      Double_t kappa2LOS ( const Double_t& zt ,const Double_t& c2w, const Double_t& mz, 
                           const Double_t& mu, const Double_t& sman, const Double_t& i3qf);
      Double_t deltaeta  ( const Double_t& zt, const Double_t& ht, const Double_t& c2, const Double_t& mz, const Double_t& mu);
      Double_t deltaetaOS( const Double_t& zt, const Double_t& ht, const Double_t& s2, 
                           const Double_t& mz, const Double_t& mu, const Double_t& sman, const Double_t& i3qf);

      Bool_t m_logMH;
      
   private:

      // boolean flags to track recomputation
      Bool_t m_isUpToDate_Update; 

      ClassDef(ZFitterEW2Loop,0)
   };
}

//--------------------------------inline functions--------------------------------------

inline Double_t GSM::ZFitterEW2Loop::GetDrho2L()
{
   Update();
   return m_Drho2L;
}

inline Double_t GSM::ZFitterEW2Loop::GetDkap2L()
{
   Update();
   return m_Dkap2L;
}

inline Double_t GSM::ZFitterEW2Loop::GetDrdRem()
{
   Update();
   return m_DrdRem;
}

inline Double_t GSM::ZFitterEW2Loop::GetDkdRem( Double_t pi3qf )
{
   Update();
   Double_t kapOS = kappa2LOS(m_zt, m_c2, m_MZ, m_mu, m_sman, pi3qf);

   return m_Nc*m_xt*m_xt*(m_kappa - 4.0*m_c2*m_c2*m_zt*kapOS);
}

inline Double_t GSM::ZFitterEW2Loop::GetRhoRem( Double_t pi3qf )
{
   Update();
  
   Double_t eta =  ( 4.0*m_deltaeta + deltaetaOS(m_zt, m_ht, m_s2, m_MZ, m_mu, m_sman, pi3qf) 
                     - 8.0*m_c2*log(m_c2) );
  
   return ( m_Nc*m_xt*m_xt
            *(4.0*m_zt*m_c2*eta - (16.0*m_deltarw + 4.0*m_zt*m_c2*m_deltarwOS)) );
}

#endif
