/**********************************************************************************
 * Project: GSM - Electroweak fitting package                                     *
 * Package: GSM                                                                   *
 * Class  : ZFitterFermionPart                                                    *
 *                                                                                *
 * Description:                                                                   *
 *      Auxiliary Theory for fermionic part of self energies                      *
 *      one loop core of ZFitter option                                           *
 *                                                                                *
 * Source:                                                                        *
 *      hep-ph/9709299, hep-ph/9908433                                            *
 *      ZFitter package dizet6_42.f                                               *
 *                                                                                *
 * Additional Information                                                         *
 *      The Standard Model in the Making, Oxford 1999                             *
 *      Nucl. Phys. B197 (1982) 1-44 / first summary of one loop core             *
 *                                                                                *
 * Authors:                                                                       *
 *      See gsm/AUTHORS file                                                      *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: ZFitterFermionPart.h,v 1.12 2008/05/18 15:51:12 mgoebel Exp $
 **********************************************************************************/
#ifndef GSM_ZFitterFermionPart
#define GSM_ZFitterFermionPart

#include <complex>
#include "Gfitter/GAuxTheory.h"

class Gfitter::GReference;
class Gfitter::GParameterRef;
class Gfitter::GTheoryRef;

namespace GSM {

   class QPoleMass;
      
   class ZFitterFermionPart : public Gfitter::GAuxTheory {
          
   public:
     
      ZFitterFermionPart();
      virtual ~ZFitterFermionPart() {}

      // accessors, downcast for using them
      std::complex<Double_t> GetWfAt0();
      std::complex<Double_t> GetZfAt0();
      std::complex<Double_t> GetWfAtMW();
      std::complex<Double_t> GetZfAtMZ();
      std::complex<Double_t> GetWfFAtMW();
      std::complex<Double_t> GetZfFAtMZ();
      std::complex<Double_t> GetMfPhoZAtMZ();
      
      //not used
      void Initialise() {}
      
   protected:

      // update of flags in case of parameter change
      void UpdateLocalFlags( Gfitter::GReference& ref );
   
      // update parameters
      void Update();
 
      // reference to W mass
      Gfitter::GTheory& GetMW() const { return (Gfitter::GTheory&)*t_MW; }      
      Gfitter::GTheoryRef    t_MW; 
     
      // reference to pole quark masses
      QPoleMass& GetQPoleMass() const { return (QPoleMass&)*t_QPoleMass; }      
      Gfitter::GTheoryRef    t_QPoleMass;  
      
      // Data-Card members
      Gfitter::GParameterRef p_MZ; 
      Gfitter::GParameterRef p_mt; 
      

      // updated variables
      Double_t m_MW2;
      Double_t m_MZ2;
     
      Double_t m_R;
      Double_t m_R1;
      Double_t m_R12;

      // member variables
      // self energies etc.
      std::complex<Double_t> m_WfAt0;
      std::complex<Double_t> m_ZfAt0;
      std::complex<Double_t> m_WfAtMW;
      std::complex<Double_t> m_ZfAtMZ;
      std::complex<Double_t> m_ZfFAtMZ;
      std::complex<Double_t> m_WfFAtMW;
      std::complex<Double_t> m_MfPhoZAtMZ;

      // lepton masses
      Double_t m_ml[2][3];
      // quark masses
      Double_t m_mq[2][3];
      // quark charges
      Double_t m_chq[2];

      // functions
      std::complex<Double_t> MfphoZ( const Double_t& Q2 ) const;
      std::complex<Double_t> DerivativeWF( const Double_t& Q2 ) const;
      std::complex<Double_t> DerivativeZF( const Double_t& Q2 ) const;

   private:
    
      // boolean flags to track recomputation
      Bool_t m_isUpToDate_Update; 

      ClassDef(ZFitterFermionPart,0) 
   };
}
//--------------------- inline functions for fast access --------------------

// W boson self energy
inline std::complex<Double_t> GSM::ZFitterFermionPart::GetWfAt0()
{ 
   Update();
   return m_WfAt0;
}

inline std::complex<Double_t> GSM::ZFitterFermionPart::GetWfAtMW()
{
   Update();
   return m_WfAtMW; 
}

// Z boson self energy
inline std::complex<Double_t> GSM::ZFitterFermionPart::GetZfAt0()
{
   Update();
   return m_ZfAt0; 
}

inline std::complex<Double_t> GSM::ZFitterFermionPart::GetZfAtMZ()
{ 
   Update();
   return m_ZfAtMZ;
}

// derivatives of self energies / F = finite
inline std::complex<Double_t> GSM::ZFitterFermionPart::GetZfFAtMZ()
{
   Update();
   return m_ZfFAtMZ; 
}

inline std::complex<Double_t> GSM::ZFitterFermionPart::GetWfFAtMW()
{ 
   Update();
   return m_WfFAtMW;
}

// photon Z mixing function
inline std::complex<Double_t> GSM::ZFitterFermionPart::GetMfPhoZAtMZ()
{
   Update();
   return m_MfPhoZAtMZ;
}

#endif
