/**********************************************************************************
 * Project: GSM - Electroweak fitting package                                     *
 * Package: GSM                                                                   *
 * Class  : ZFitterQCDCorrection                                                  *
 *                                                                                *
 * Description:                                                                   *
 *      2 loopQCD Corrections to the effective weak couplings                     *
 *      in perturbative QCD                                                       *
 *      Nucl. Phys. B347 (1990) 86-104 by Kniehl                                  *
 *      Translation of Kniehl's bkqcd5_14.f package from ZFitter                  *
 *                                                                                *
 * Authors:                                                                       *
 *      See gsm/AUTHORS file                                                      *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: ZFitterQCDCorrections.cxx,v 1.11 2007/10/24 09:46:28 mgoebel Exp $
 **********************************************************************************/

#include "TMath.h"

#include "Gfitter/GMath.h"
#include "Gfitter/GTheory.h"
#include "Gfitter/GTheoryRef.h"
#include "Gfitter/GParameterRef.h"
#include "Gfitter/GReference.h"

#include "GSM/ZFitterQCDCorrections.h"
#include "GSM/ZFitterMath.h"

using namespace Gfitter;
using std::complex;

GSM::ZFitterQCDCorrections::ZFitterQCDCorrections()
   : Gfitter::GAuxTheory(),
     m_isUpToDate_Update( kFALSE )
{
   SetTheoryName( GetName() );
   SetExistDerivative( kFALSE );
  
   BookParameter( "MZ"                 , & p_MZ );
   BookParameter( "mt"                 , & p_mt );
   BookTheory   ( "GSM::MW"            , & t_MW );
}
  
void GSM::ZFitterQCDCorrections::Initialise()
{    
   m_cone  = 1.0;
   m_zeta3 = GMath::Zeta3();
}

void GSM::ZFitterQCDCorrections::UpdateLocalFlags( GReference& /* ref */ )
{
   m_isUpToDate_Update = kFALSE;
}

void GSM::ZFitterQCDCorrections::Update()
{
   if (m_isUpToDate_Update) return;

   // now, it is uptodate (I mean... it will be)
   m_isUpToDate_Update = kTRUE;

   m_kmQCD = KMQCD( p_MZ*p_MZ, GMath::IPow( GetMW(),2 ), p_mt*p_mt, p_MZ*p_MZ );
   m_rmQCD = RMQCD( p_MZ*p_MZ, GMath::IPow( GetMW(),2 ), p_mt*p_mt, p_MZ*p_MZ );
     
   SetUpToDate();
}

// ------------------------ additional part from ZFITTER Authors ------------------------------

// Correction to kappa_f
complex<Double_t> GSM::ZFitterQCDCorrections::KMQCD( const Double_t& MZ2, const Double_t& MW2, 
                                                     const Double_t& mt2, const Double_t& S ) const
{    
   Double_t CW2 = MW2/MZ2;
   Double_t SW2 = 1.0 - CW2;
   Double_t QTM = 2/3.0;
   Double_t QBM = 1/3.0;
   complex<Double_t> XCR = MZ2/(4.0*mt2);
   complex<Double_t> XCS = S/(4.0*mt2);
   complex<Double_t> XCX = MW2/mt2;
   Double_t VT2 = GMath::IPow(1.0 - 4.0*QTM*SW2,2);

   complex<Double_t> kmqcd = ( CW2/(SW2*SW2)/4.0*mt2/MW2*(VT2*CV1(XCR) + CA1(XCR))
                               - CW2/(SW2*SW2)*mt2/MW2*CF1(XCX)
                               + 1.0/SW2*(1.0-4.0*QTM*SW2)*QTM*mt2/S*CV1(XCS)
                               - 1.0/(16.0*SW2*SW2)*(2.0*(1.0-2.0*QBM*SW2)*TMath::Log(MZ2/mt2)
                                                     + 4.0*(1.0-4.0*QBM*SW2)*QBM*SW2*TMath::Log(S/MZ2)) );
   return kmqcd;
}

// Correction to rho_f
complex<Double_t> GSM::ZFitterQCDCorrections::RMQCD( const Double_t& MZ2, const Double_t& MW2, 
                                                     const Double_t& mt2, const Double_t& S ) const
{    
   Double_t CW2  = MW2/MZ2;
   Double_t SW2  = 1.0 - CW2;
   Double_t SMZ2 = S/MZ2;
   Double_t DMZ2 = 1.0 - SMZ2;
   Double_t QTM  = 2/3.0;
   Double_t QBM  = 1/3.0;
   Double_t D2 = 1.6449340668482;
   Double_t D3 = 1.2020569031596;

   Double_t VT2 = GMath::IPow(1.0 - 4.0*QTM*SW2,2);
   Double_t VB2 = GMath::IPow(1.0 - 4.0*QBM*SW2,2);
    
   complex<Double_t> rmqcd(0.0,0.0);

   if (TMath::Abs(DMZ2) < 1e-3) {
          
      Double_t ALTZ = -mt2/MZ2;
      
      complex<Double_t> PVFTZ = ZFitterMath::PolVfi(ALTZ);
      complex<Double_t> PAFTZ = ZFitterMath::PolAfi(ALTZ);
      
      complex<Double_t> DVFTZ = ZFitterMath::DPolVfi(ALTZ);
      complex<Double_t> DAFTZ = ZFitterMath::DPolAfi(ALTZ);
      rmqcd = 1.0/(12.0*SW2)*( 3.0/(4.0*CW2)*(1.0 + VB2)
                               + mt2/(4.0*MW2)*(VT2*(DVFTZ/ALTZ - PVFTZ)
                                                + DAFTZ/ALTZ - PAFTZ)
                               - mt2/MW2*(3.0*D2 +105/8.0) );
   }
   else {
      complex<Double_t> XCR   = MZ2/(4.0*mt2);
      complex<Double_t> XCS   = S/(4.0*mt2);
      complex<Double_t> XV1r  = CV1(XCR);
      complex<Double_t> XA1r  = CA1(XCR);
      complex<Double_t> XV1rs = CV1(XCS);
      complex<Double_t> XA1rs = CA1(XCS);
      
      rmqcd = 1.0/(4.0*SW2*CW2)*( mt2/MZ2*(VT2*XV1r + XA1r)
                                  + mt2/(MZ2-S)*(VT2*(XV1rs - XV1r) + XA1rs - XA1r)
                                  + 2.0*mt2/MZ2*(-23.0/8.0*D2+3.0*D3)
                                  - 1.0/4.0*(1.0 + QBM)*S/(MZ2-S)*TMath::Log(S/MZ2) );
   }

   return rmqcd;
}

// ------------------------ Kniehl's part starts here ------------------------------


// V_1(r) defined by Eq. (13) in Nucl. Phys. B347 (1990)
complex<Double_t> GSM::ZFitterQCDCorrections::CV1( const complex<Double_t>& r ) const
{    
   complex<Double_t> cv1(0.0,0.0);
   complex<Double_t> I  (0.0,1.0);
   
   if      (r == cv1) cv1 = 0.0;
   else if (r == m_cone) {
      m_logger << kWARNING <<  "Warning: Coulomb singularity of V_1(r) at r = 1 !" << GEndl ;
      cv1 = I*TMath::Pi()*TMath::Pi()*TMath::Pi();
   }
   else {
      complex<Double_t> C1  = GMath::CSqrt(m_cone-r,-m_cone);
      complex<Double_t> C2  = GMath::CSqrt(-r,-m_cone);
      complex<Double_t> CS  = C1/C2;
      complex<Double_t> CP  = C1 + C2;
      complex<Double_t> CM  = C1 - C2;
      complex<Double_t> CMS = CM*CM;
      complex<Double_t> CMQ = CMS*CMS;
      complex<Double_t> CF  = GMath::Log(CP);
      complex<Double_t> CFS = CF*CF;
      complex<Double_t> CG  = GMath::Log(CP-CM);
      complex<Double_t> CH  = GMath::Log(CP+CM);
      complex<Double_t> CD  = GMath::Li2(CMS,m_cone) - GMath::Li2(CMQ,m_cone);
      
      cv1 = ( 4.0*(r - 1.0/(4.0*r))*( 2.0*GMath::Li3(CMS,m_cone) 
                                      - GMath::Li3(CMQ,m_cone) + 8.0/3.0*CF*CD
                                      + 4.0*CFS*(-CF+CG/3.0 + 2.0/3.0*CH) )
              + CS*(8.0/3.0*(r + 1.0/2.0)*(CD + CF*(-3.0*CF+2.0*CG+4.0*CH))-2.0*(r+3.0/2.0)*CF)
              - 8.0*(r-1.0/6.0-7.0/(48.0*r))*CFS+13.0/6.0 + m_zeta3/r );
   }

   return cv1;
}

// A_1(r) defined by Eq. (14) in Nucl. Phys. B347 (1990)
complex<Double_t> GSM::ZFitterQCDCorrections::CA1( const complex<Double_t>& r ) const
{   
    
   complex<Double_t> ca1(0.0,0.0);
    
   if      (r == ca1)    ca1 = 3.0*(-2.0*m_zeta3 - GMath::Zeta2() + 7.0/4.0);
   else if (r == m_cone) ca1 = - 2.0*m_zeta3 -3.0/8.0*GMath::Zeta2() + 29.0/12.0;
   else {
      complex<Double_t> C1  = GMath::CSqrt(m_cone-r,-m_cone);
      complex<Double_t> C2  = GMath::CSqrt(-r,-m_cone);
      complex<Double_t> CS  = C1/C2;
      complex<Double_t> CP  = C1 + C2;
      complex<Double_t> CM  = C1 - C2;
      complex<Double_t> CMS = CM*CM;
      complex<Double_t> CMQ = CMS*CMS;
      complex<Double_t> CF  = GMath::Log(CP);
      complex<Double_t> CFS = CF*CF;
      complex<Double_t> CG  = GMath::Log(CP-CM);
      complex<Double_t> CH  = GMath::Log(CP+CM);
      complex<Double_t> CD  = GMath::Li2(CMS,m_cone) - GMath::Li2(CMQ,m_cone);
      ca1 = ( 4.0*(r - 3.0/2.0 + 1.0/(2.0*r))*( 2.0*GMath::Li3(CMS,m_cone)
                                                - GMath::Li3(CMQ,m_cone) + 8.0/3.0*CF*CD
                                                + 4.0*CFS*(-CF+CG/3.0+2.0/3.0*CH) )
              + CS*(8.0/3.0*(r - 1.0)*(CD + CF*(-3.0*CF+2.0*CG+4.0*CH)) - 2.0*(r - 3.0 + 1.0/(4.0*r))*CF)
              - 8.0*(r - 11.0/12.0 + 5.0/(48.0*r) + 1.0/(32.0*r*r))*CFS
              - 3.0*GMath::Zeta2()+13.0/6.0+(-2.0*m_zeta3+1.0/4.0)/r );
   }

   return ca1;
}

// F_1(r) defined by Eq. (15) in Nucl. Phys. B347 (1990)  
complex<Double_t> GSM::ZFitterQCDCorrections::CF1( const complex<Double_t>& x ) const
{   
   complex<Double_t> cf1 (0.0,0.0);
    
   if      (x == cf1)    cf1 = -3.0/2.0*m_zeta3 - GMath::Zeta2()/2.0 + 23.0/16.0;
   else if (x == m_cone) cf1 = -m_zeta3/2.0-GMath::Zeta2()/12.0 + 7.0/8.0;
   else {
      complex<Double_t> xS  = x*x;
      complex<Double_t> CA  = GMath::CLog(-x,-m_cone);
      complex<Double_t> CB  = GMath::CLog(m_cone-x,-m_cone);
      complex<Double_t> CBS = CB*CB;
      complex<Double_t> CD  = GMath::Li2(m_cone/(m_cone-x),m_cone);
    
      cf1 = ( (x-3.0/2.0+1.0/(2.0*xS))*( GMath::Li3(m_cone/(m_cone-x),m_cone)
                                         + 2.0/3.0*CB*CD-CBS/6.0*(CA-CB) )
              + (x+1.0/2.0-1.0/(2.0*x))/3.0*(CD-CA*CB)
              + (x-1.0/8.0-1.0/x+5.0/(8.0*xS))/3.0*CBS
              - (x-5.0/2.0+2.0/(3.0*x)+5.0/(6.0*xS))/4.0*CB
              - 3.0/4.0*GMath::Zeta2()+13.0/12.0-5.0/(24.0*x)
              - m_zeta3/(2.0*xS) );
   }
   
   return cf1;
}
  
